/* Teljes merevlemez */
#include <ctype.h>
#include <errno.h>
#include <fcntl.h>
#include <stdint.h>
#include <dirent.h>
#include <syscall.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <scsi/scsi.h>
#include <unistd.h>

#include "partition.h"
#include "installer.h"

#ifndef SCSI_IOCTL_SEND_COMMAND
#define SCSI_IOCTL_SEND_COMMAND 1
#endif

/* from <linux/hdreg.h> */
#define HDIO_GET_IDENTITY	0x030d	/* get IDE identification info */



/* structure returned by HDIO_GET_IDENTITY, as per ANSI ATA2 rev.2f spec */
struct hd_driveid {
	unsigned short config;		/* lots of obsolete bit flags */
	unsigned short cyls;		/* "physical" cyls */
	unsigned short reserved2;	/* reserved (word 2) */
	unsigned short heads;		/* "physical" heads */
	unsigned short track_bytes;	/* unformatted bytes per track */
	unsigned short sector_bytes;	/* unformatted bytes per sector */
	unsigned short sectors;		/* "physical" sectors per track */
	unsigned short vendor0;		/* vendor unique */
	unsigned short vendor1;		/* vendor unique */
	unsigned short vendor2;		/* vendor unique */
	unsigned char serial_no[20];	/* 0 = not_specified */
	unsigned short buf_type;
	unsigned short buf_size;	/* 512 byte increments; 0 = not_specified */
	unsigned short ecc_bytes;	/* for r/w long cmds; 0 = not_specified */
	unsigned char fw_rev[8];	/* 0 = not_specified */
	unsigned char model[40];	/* 0 = not_specified */
	unsigned char max_multsect;	/* 0=not_implemented */
	unsigned char vendor3;		/* vendor unique */
	unsigned short dword_io;	/* 0=not_implemented; 1=implemented */
	unsigned char vendor4;		/* vendor unique */
	unsigned char capability;	/* bits 0:DMA 1:LBA 2:IORDYsw 3:IORDYsup */
	unsigned short reserved50;	/* reserved (word 50) */
	unsigned char vendor5;		/* vendor unique */
	unsigned char tPIO;			/* 0=slow, 1=medium, 2=fast */
	unsigned char vendor6;		/* vendor unique */
	unsigned char tDMA;			/* 0=slow, 1=medium, 2=fast */
	unsigned short field_valid;	/* bits 0:cur_ok 1:eide_ok */
	unsigned short cur_cyls;	/* logical cylinders */
	unsigned short cur_heads;	/* logical heads */
	unsigned short cur_sectors;	/* logical sectors per track */
	unsigned short cur_capacity0;	/* logical total sectors on drive */
	unsigned short cur_capacity1;	/*  (2 words, misaligned int)     */
	unsigned char multsect;		/* current multiple sector count */
	unsigned char multsect_valid;	/* when (bit0==1) multsect is ok */
	unsigned int lba_capacity;	/* total number of sectors */
	unsigned short dma_1word;	/* single-word dma info */
	unsigned short dma_mword;	/* multiple-word dma info */
	unsigned short eide_pio_modes;	/* bits 0:mode3 1:mode4 */
	unsigned short eide_dma_min;	/* min mword dma cycle time (ns) */
	unsigned short eide_dma_time;	/* recommended mword dma cycle time (ns) */
	unsigned short eide_pio;	/* min cycle time (ns), no IORDY  */
	unsigned short eide_pio_iordy;	/* min cycle time (ns), with IORDY */
	unsigned short words69_70[2];	/* reserved words 69-70 */
	/* HDIO_GET_IDENTITY currently returns only words 0 through 70 */
	unsigned short words71_74[4];	/* reserved words 71-74 */
	unsigned short queue_depth;	/*  */
	unsigned short words76_79[4];	/* reserved words 76-79 */
	unsigned short major_rev_num;	/*  */
	unsigned short minor_rev_num;	/*  */
	unsigned short command_set_1;	/* bits 0:Smart 1:Security 2:Removable 3:PM */
	unsigned short command_set_2;	/* bits 14:Smart Enabled 13:0 zero */
	unsigned short cfsse;		/* command set-feature supported extensions */
	unsigned short cfs_enable_1;	/* command set-feature enabled */
	unsigned short cfs_enable_2;	/* command set-feature enabled */
	unsigned short csf_default;	/* command set-feature default */
	unsigned short dma_ultra;	/*  */
	unsigned short word89;		/* reserved (word 89) */
	unsigned short word90;		/* reserved (word 90) */
	unsigned short CurAPMvalues;	/* current APM values */
	unsigned short word92;		/* reserved (word 92) */
	unsigned short hw_config;	/* hardware config */
	unsigned short words94_125[32];	/* reserved words 94-125 */
	unsigned short last_lun;	/* reserved (word 126) */
	unsigned short word127;		/* reserved (word 127) */
	unsigned short dlf;			/* device lock function
								   * 15:9   reserved
								   * 8  security level 1:max 0:high
								   * 7:6    reserved
								   * 5  enhanced erase
								   * 4  expire
								   * 3  frozen
								   * 2  locked
								   * 1  en/disabled
								   * 0  capability
								 */
	unsigned short csfo;		/* current set features options
								   * 15:4   reserved
								   * 3  auto reassign
								   * 2  reverting
								   * 1  read-look-ahead
								   * 0  write cache
								 */
	unsigned short words130_155[26];	/* reserved vendor words 130-155 */
	unsigned short word156;
	unsigned short words157_159[3];	/* reserved vendor words 157-159 */
	unsigned short words160_255[95];	/* reserved words 160-255 */
};

int db = 0;
int partdb = 0; 

/* functions ------------------------------------------------------------------------------------------*/

char *strip_name(char **str)
{
	int i;
	int end = 0;

	for (;isspace(**str);(*str)++);

	for (i = 0; (*str)[i]; i++) {
		if (!isspace((*str)[i])
			|| (isspace((*str)[i]) && !isspace((*str)[i + 1]) && (*str)[i + 1])) {
			(*str)[end] = (*str)[i];
			end++;
		}
	}
	(*str)[end] = 0;
	return strdup(*str);
}

char *get_model(int fd)
{
	struct hd_driveid hdi;
	char hdi_buf[41];

	/* The following are defined by the SCSI-2 specification. */
	typedef struct _scsi_inquiry_cmd {
		uint8_t op;
		uint8_t lun;			/* bits 5-7 denote the LUN */
		uint8_t page_code;
		uint8_t reserved;
		uint8_t alloc_length;
		uint8_t control;
	} __attribute__ ((packed)) scsi_inquiry_cmd_t;

	typedef struct _scsi_inquiry_data {
		uint8_t peripheral_info;
		uint8_t device_info;
		uint8_t version_info;
		uint8_t _field1;
		uint8_t additional_length;
		uint8_t _reserved1;
		uint8_t _reserved2;
		uint8_t _field2;
		uint8_t vendor_id[8];
		uint8_t product_id[16];
		uint8_t product_revision[4];
		uint8_t vendor_specific[20];
		uint8_t _reserved3[40];
	} __attribute__ ((packed)) scsi_inquiry_data_t;

	struct scsi_arg {
		unsigned int inlen;
		unsigned int outlen;

		union arg_data {
			scsi_inquiry_data_t out;
			scsi_inquiry_cmd_t in;
		} data;
	} arg;

	struct scsi_idlun {
		uint32_t dev_id;
		uint32_t host_unique_id;
	} idlun;

	if (!ioctl(fd, HDIO_GET_IDENTITY, &hdi)) {
		char *model;
		
		/* hdi.model is not guaranteed to be NULL terminated */
		memcpy(hdi_buf, hdi.model, 40);
		hdi_buf[40] = '\0';
		model = hdi.model;
		return strip_name(&model);
	}

	if (ioctl(fd, SCSI_IOCTL_GET_IDLUN, &idlun) >= 0) {
		memset(&arg, 0x00, sizeof(struct scsi_arg));
		arg.inlen = 0;
		arg.outlen = sizeof(scsi_inquiry_data_t);
		arg.data.in.op = INQUIRY;
		arg.data.in.lun = idlun.host_unique_id << 5;
		arg.data.in.alloc_length = sizeof(scsi_inquiry_data_t);
		arg.data.in.page_code = 0;
		arg.data.in.reserved = 0;
		arg.data.in.control = 0;

		if (ioctl(fd, SCSI_IOCTL_SEND_COMMAND, &arg) >= 0) {
			char buf[32];
			char *vendor;
			char *product;
			char *model;

			memcpy(buf, arg.data.out.vendor_id, 8);
			buf[8] = '\0';
			vendor = buf;
			vendor = strip_name(&vendor);

			memcpy(buf, arg.data.out.product_id, 16);
			buf[16] = '\0';
			product = buf;
			product = strip_name(&product);

			model = (char *) malloc(8 + 16 + 2);

			if (model) {
				sprintf(model, "%s%s%s", vendor?vendor:"", (vendor&&vendor[0]&&product&&product[0])?" ":"", product?product:"");
				
				if (vendor)
					free(vendor);
				if (product)
					free(product);

				return model;
			}
		}
	}

	return NULL;
}

int getdevs() {
	char discs[100], *dname, tmphdd[120];
	DIR *dir;
	struct dirent *direntp;
	int j = 0;
	
	/* nullazas */
	db = 0;

	if (! (dir = opendir("/dev/discs/")) ) { 
	 popupmsg("Hiba","Nem tudtam megnyitni a /dev/discs konvtarat!\n\n"); off(); 
	}
	
	while ((direntp = readdir(dir)) ) {
	 dname = direntp->d_name;
		 	 	 
	  if ( !(strstr(dname,".")) ) {
	  sprintf(discs,"/dev/discs/%s",dname);
	 
	  if ( (realpath (discs,tmphdd)) != NULL) {
	   sprintf(hddparts[j].hdddev,"%s/disc",tmphdd);
	   j++;
	   db++; 
	  } else { popupmsg("Hiba","Nem tudtam feloldani a valos elerest!\n\n"); off(); }
	 }
	}
	
	if ( (closedir(dir)) ) { popupmsg("Hiba","Nem tudtam closdirt() vegrehajtani!\n\n"); }
	return(0);	
}

int getsize () {
	int fd = 0, b, u, o;
	int size;
	
  for (u = 0; u<db; u++) {
	 
	if ( (fd = open(hddparts[u].hdddev, O_RDONLY | O_NONBLOCK)) ) {
	 if ( (b = ioctl(fd,BLKGETSIZE, &size)) == 0) {	
	  size = ( (size / 1024) / 2 );
	   if ( size < 1000 ) { 
	      sprintf(hddparts[u].hddsize,"%d(M)",size); 			   
	   } else { 
	     size = size / 1024; 
		 sprintf(hddparts[u].hddsize,"%d(G)",size);	   
	   }
	 }

	} else { perror("device"); }
	
  } close(fd); 
  
  for (u = 0; u<db; u++) {
   for (o = 0; o<hddparts[u].partnum; o++) {
	 
	if ( (fd = open(hddparts[u].part[o], O_RDONLY | O_NONBLOCK)) ) {
	 if ( (b = ioctl(fd,BLKGETSIZE, &size)) == 0) {	
	  size = ( (size / 1024) / 2 );
	   if ( size < 1100 ) { 
	      sprintf(hddparts[u].partsize[o],"%d(M)",size); 			   
	   } else { 
	     size = size / 1024; 
		 sprintf(hddparts[u].partsize[o],"%d(G)",size);	   
	   }
	 }

	} else { perror("device"); }
	
   }	
  } close(fd);
  
  return(0);
}

int getmodelsname() {
	
 	int fd = 0, i = 0;
	char *model;
	
	while (1) {
	
	if ( !(strlen(hddparts[i].hdddev)) ) { break; }
	
	if ((fd = open(hddparts[i].hdddev, O_RDONLY | O_NONBLOCK)) < 0) {
		fprintf(stderr, "getmodel: %s: %s\n", hddparts[i].hdddev, strerror(errno));
		exit(1); 
	}

	if (!(model = get_model(fd))) return(1); 
     
	 sprintf(hddparts[i].hddname,"%s",model); 
	 
     i++;
 } close(fd);
 
 return(0);

}

int getswap() {
	struct stat buf;
	int temp;
		
	if ( stat("/proc/kcore", &buf) != -1 ) {
	 
	 if ( buf.st_size < 256 ) {
	  temp = ((buf.st_size / 1024) / 1024) * 2;
	 } else { temp = 256; }
	  
	 sprintf(swapmem,"%d",temp); 
 	}
	
	return(0);
}

int getparts(int i) {
	
	FILE *file;
	char tmp1[120],devtmp[120];
	char *a,*b,*c,*d,*e,*f;
	int j = 0;
	
	sprintf(devtmp,"fdisk -l %s",hddparts[i].hdddev);
		
	file = popen(devtmp,"r");
	while ( (fgets(tmp1,120,file)) != NULL) {
	 	 
	 if ( (strstr(tmp1,"part")) != NULL ) { 
	  sscanf(tmp1,"%s",hddparts[i].part[j]);
	  realpath(hddparts[i].part[j],hddparts[i].part[j]); 
	   if ( (strstr(tmp1,"*")) ) {
	    sscanf(tmp1,"%s %[*] %s %s %s %s %[^\n]",&a,&b,&c,&d,&e,&f,hddparts[i].parttype[j]);
	   } else {
	    sscanf(tmp1,"%s %s %s %s %s %[^\n]",&a,&b,&c,&d,&e,hddparts[i].parttype[j]);
	   }
	     
		j++; partdb++;		
	 }
	 
	} hddparts[i].partnum = j; 
	
	pclose(file);
	
	return(0);
}
